// gui_pad_free.ino
// ツール -> ESP32 sketch data upload
#include <WiFi.h>
#include <ESPUI.h>
#include <FastLED.h>

// LED 16bit
const int DATA_PIN = 19;
const int NUM_LEDS = 16;
CRGB leds[NUM_LEDS];

// WiFi
const char *ssid = "tokushima-01";
const char *password = "12345678";
const IPAddress ip(192, 168, 4, 1);
const IPAddress subnet(255, 255, 255, 0);

// LED
const int LED_PIN = 18;

// Buzzer
const int SPEAKER_PIN = 15;              // buzzer pin
const int CH_2        = 2;    // use first channel of 16 channels (started from zero)
const int CH_2_TIMER  = 13;   // use 13 bit precission for LEDC timer
const int CH_2_FREQ   = 5000; // use 5000 Hz as a LEDC base frequency

// モーターの変数 phase/enable mode
const int MOTOR_L_CWCCW = 16; // APH direction
const int MOTOR_R_CWCCW = 17; // BPH direction
const int MOTOR_L_PWM   = 26; // AEN pwm
const int MOTOR_R_PWM   = 4; // BEN pwm

// PWM
const int CH_0 = 0;
const int CH_1 = 1;
const int LEDC_TIMER_BIT = 10;   // PWMの範囲(8bitなら0〜255、10bitなら0〜1023)
const int LEDC_BASE_FREQ = 120000; // 周波数(Hz) 50khz
const int VALUE_MAX = 1023;      // PWMの最大値
const int sp = 600; // モーター左右のスピード: 0 - 1023

void buttonA_Callback(Control sender, int type) {
  switch (type) {
    case B_DOWN:
      Serial.println("Button A DOWN");
      ledcWriteTone(CH_2, 600);
      delay(150);
      ledcWriteTone(CH_2, 800);
      delay(300);
      ledcWriteTone(CH_2, 0);
      break;
  }
}

void buttonB_Callback(Control sender, int type) {
  switch (type) {
    case B_DOWN:
      Serial.println("Button B DOWN");
      for (int i = 0; i < 3; i++) {
        fill_solid(leds, NUM_LEDS, CRGB(255, 0, 255));
        FastLED.show();
        delay(500);
        fill_solid(leds, NUM_LEDS, CRGB(0, 0, 0));
        FastLED.show();
        delay(500);
      }
      //      digitalWrite(LED_PIN, HIGH);
      //      delay(1000);
      //      digitalWrite(LED_PIN, LOW);
      break;
  }
}

void padExample(Control sender, int value) {
  switch (value) {
    case P_LEFT_DOWN:
      leds[8] = CRGB(255, 0, 255);
      motor(sp, sp, LOW, HIGH);
      break;
    case P_LEFT_UP:
      leds[8] = CRGB(0, 0, 0);
      motor(sp / 2, sp / 2, HIGH, HIGH);
      delay(10);
      motor(0, 0, HIGH, HIGH);
      break;

    case P_RIGHT_DOWN:
      leds[0] = CRGB(0, 255, 255);
      motor(sp, sp, HIGH, LOW);
      break;
    case P_RIGHT_UP:
      leds[0] = CRGB(0, 0, 0);
      motor(sp / 2, sp / 2, HIGH, HIGH);
      delay(10);
      motor(0, 0, LOW, HIGH);
      break;

    case P_FOR_DOWN:
      leds[12] = CRGB(255, 255, 0);
      motor(sp, sp, HIGH, HIGH);
      break;
    case P_FOR_UP:
      leds[12] = CRGB(0, 0, 0);
      motor(sp / 2, sp / 2, HIGH, HIGH);
      delay(10);
      motor(0, 0, LOW, LOW);
      break;

    case P_BACK_DOWN:
      leds[4] = CRGB(255, 0, 255);
      motor(sp, sp, LOW, LOW);
      break;
    case P_BACK_UP:
      leds[4] = CRGB(0, 0, 0);
      motor(0, 0, LOW, LOW);
      break;

    case P_CENTER_DOWN:
      static int h;
      fill_solid(leds, NUM_LEDS, CHSV(h, 255, 255));
      h += 10;
      if (h > 255) {
        fill_solid(leds, NUM_LEDS, CRGB(0, 0, 0));
        h = 0;
      }
      FastLED.show();
      break;
    default:
      motor(sp / 2, sp / 2, HIGH, HIGH);
      delay(10);
      motor(0, 0, HIGH, HIGH);
      break;
  }
  leds[12] = CRGB(0, 0, 255);
  FastLED.show();
}

void setup(void) {
  Serial.begin(115200);
  delay(100);

  WiFi.disconnect(true);
  delay(1000);
  WiFi.softAP(ssid, password);
  delay(100);
  WiFi.softAPConfig(ip, ip, subnet);
  IPAddress myIP = WiFi.softAPIP();

  // led
  pinMode(LED_PIN, OUTPUT);

  // モータのポート設定
  pinMode(MOTOR_L_CWCCW, OUTPUT);
  pinMode(MOTOR_R_CWCCW, OUTPUT);
  ledcSetup(CH_0, LEDC_BASE_FREQ, LEDC_TIMER_BIT);
  ledcSetup(CH_1, LEDC_BASE_FREQ, LEDC_TIMER_BIT);
  ledcAttachPin(MOTOR_L_PWM, CH_0);
  ledcAttachPin(MOTOR_R_PWM, CH_1);
  ledcWrite(CH_0, 0);
  ledcWrite(CH_1, 0);

  // speaker
  ledcSetup(CH_2, CH_2_FREQ, CH_2_TIMER);
  ledcAttachPin(SPEAKER_PIN, CH_2);

  // fastled
  FastLED.addLeds<WS2812B, DATA_PIN, RGB>(leds, NUM_LEDS);
  FastLED.setBrightness(16);
  set_max_power_in_volts_and_milliamps(5, 100);
  fill_solid(leds, NUM_LEDS, CRGB(0, 0, 0));
  leds[12] = CRGB(0, 0, 255);
  FastLED.show();

  ESPUI.pad("Pad with center", true, &padExample, COLOR_SUNFLOWER);
  ESPUI.button("Button A", &buttonA_Callback, COLOR_PETERRIVER);
  ESPUI.button("Button B", &buttonB_Callback, COLOR_PETERRIVER);

  ESPUI.begin("ESP32 Control");
}

void loop() {
}

//===========================================
// モーターの関数
//===========================================
int left_offset = 0;
int right_offset = 0;
void motor(int left, int right, int left_c, int right_c) {
  digitalWrite(MOTOR_L_CWCCW, left_c);
  digitalWrite(MOTOR_R_CWCCW, right_c);
  ledcWrite(CH_0, left + left_offset);
  ledcWrite(CH_1, right + right_offset);
}
